use rusqlite::{Connection, Error};

#[derive(Debug)]
pub enum ActionKind {
    Text,
    Script,
}

impl ActionKind {
    fn parse(kind: String) -> Result<ActionKind, Error> {
        match kind.to_lowercase().as_str() {
            "text" => Ok(ActionKind::Text),
            "script" => Ok(ActionKind::Script),
            _ => Err(Error::QueryReturnedNoRows),
        }
    }

    fn stringify(&self) -> String {
        match self {
            ActionKind::Text => "text".to_string(),
            ActionKind::Script => "script".to_string(),
        }
    }
}

#[derive(Debug)]
pub struct Action {
    id: i32,
    name: String,
    kind: ActionKind,
    source: String,
    created_at: String,
    updated_at: String,
}

pub struct Conn {
    conn: Connection,
}

impl Conn {
    pub fn new(db_path: &str) -> Result<Conn, Error> {
        let conn = Connection::open(db_path).expect("failed to open database");

        conn.execute(
            "CREATE TABLE action (
            id          INTEGER PRIMARY KEY,
            name        TEXT NOT NULL,
            kind        TEXT NOT NULL,
            source      TEXT NOT NULL,
            created_at  TEXT NOT NULL,
            updated_at  TEXT NOT NULL
        )",
            (), // empty list of parameters.
        )?;

        Ok(Conn { conn })
    }

    fn action_by_id(self, id: i32) -> Result<Action, Error> {
        let mut query = self.conn.prepare("SELECT * FROM action WHERE id = ?")?;
        let mut actions = query.query_map(&[&id], |row| {
            Ok(Action {
                id: row.get(0)?,
                name: row.get(1)?,
                kind: ActionKind::parse(row.get(2)?)?,
                source: row.get(3)?,
                created_at: row.get(4)?,
                updated_at: row.get(5)?,
            })
        })?;

        let action = actions.next();
        let len = actions.next().unwrap().iter().count();
        if len != 0 {
            return Err(Error::InvalidQuery);
        }

        if let None = action {
            return Err(Error::QueryReturnedNoRows);
        }
        action.unwrap()
    }
}
